/**
 * Copyright (c) 2009 ISP RAS.
 * 109004, A. Solzhenitsina, 25, Moscow, Russia.
 * All rights reserved.
 *
 * $Id$
 * Created on 08.10.2009
 *
 */

package mbtest.tests.account;

import mbtest.tracing.Logger;

import org.mockito.Mockito;

/**
 * @author Victor Kuliamin
 *
 */
public class AccountLogSpy
{
  static Logger logger = new Logger(AccountLogSpy.class);
  
  int balance;
  int maxCredit;
  
  Account checkedObject;
  AuditLog logSpy;
  
  public void setCheckedObject(Account checkedObject)
  {
    this.checkedObject = checkedObject;
    this.balance       = checkedObject.getBalance();
    this.maxCredit     = checkedObject.getMaxCredit();
    
    logSpy = Mockito.spy(checkedObject.getLog());
    checkedObject.setLog(logSpy);
  }

  int oldBalance;
  boolean wasPossible;
  
  public boolean possibleTransfer(int sum)
  {
    if(balance + sum > maxCredit) return true;
    else                          return false;
  }
  
  public void initSpy(int sum)
  {
    Mockito.reset(logSpy);
    oldBalance = balance;
  }
  
  public void transferLogSpy(int sum)
  {
    boolean permission = checkedObject.getPermitter().isPermittedTransfer(checkedObject, sum);

    if (wasPossible && permission)
    {
      Mockito.verify(logSpy).logKind("SUCCESS");
      Mockito.verify(logSpy).logNewBalance(balance);
    }
    else if(!permission)
      Mockito.verify(logSpy).logKind("BANNED");
    else
      Mockito.verify(logSpy).logKind("IMPROPER");
      
    Mockito.verify(logSpy).logOldBalance(oldBalance);
    Mockito.verify(logSpy).logSum(sum);
  }
  
  public void transferUpdate(int sum)
  {
    if(possibleTransfer(sum) && checkedObject.getPermitter().isPermittedTransfer(checkedObject, sum))
    {
      wasPossible = true;
      balance += sum;
    }
    else
      wasPossible = false;
  }

}
