/**
 * Copyright (c) 2009 ISP RAS.
 * 109004, A. Solzhenitsina, 25, Moscow, Russia.
 * All rights reserved.
 *
 * $Id$
 * Created on 30.09.2009
 *
 */
package mbtest.tests.account;

import mbtest.annotations.DataProvider;
import mbtest.annotations.Guard;
import mbtest.annotations.State;
import mbtest.annotations.Test;
import mbtest.tracing.Logger;

import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

/**
 * @author Victor Kuliamin
 *
 */

@Test
public class AccountTest
{
  static Logger logger = new Logger(AccountTest.class);
  
  Account account;
  boolean permission = true;
  
  @Mock Permitter permitterStub;
  
  public AccountTest()
  {
    MockitoAnnotations.initMocks(this);
    Mockito.when(permitterStub.isPermittedTransfer(Mockito.<Account>any(), Mockito.anyInt()))
           .thenReturn(true);
  }
  
  public void setAccount(Account account)       
  { 
    this.account = account;
    account.setPermitter(permitterStub);
  }

  @State
  public boolean getPermission() { return permission; }
  
  @State
  public int getBalance() 
  {
    //logger.logInfo("test      = " + this);
    logger.logDebug("object UT = " + account);
    
    return account.getBalance(); 
  }
  
  @Test
  @DataProvider(name = "sumArray")
  @Guard(names = "bound")
  public void testDeposit(int x)
  {
    account.transfer(x);
  }

  @Test(dependsOnMethods="testDeposit")
  @DataProvider(name = "sumArray")
  public void testWithdraw(int x)
  {
    account.transfer(-x);
  }
  
  @Test(dependsOnMethods="testWithdraw")
  public void switchPermission()
  {
    permission = !permission;
    Mockito.when(permitterStub.isPermittedTransfer(Mockito.<Account>any(), Mockito.anyInt()))
           .thenReturn(permission);
  }
  
  public boolean bound()
  {
    return getBalance() < 5 || !permission;
  }
  
  public int[] sumArray = new int[]{0, 1, 2};//, 3, 4, 5, 6, 7, 8, 9, 10, 11, 105, 107};
}


