/*
 * Copyright (c) 2008 ISP RAS.
 * 109004, B. Communisticheskaya, 25, Moscow, Russia.
 * All rights reserved.
 *
 * $Id$
 * Created on 01.10.2008.
 *
 */

package mbtest.tests.exploration;

import mbtest.exploration.OldBFSExplorer;
import mbtest.exploration.OldDFSExplorer;
import mbtest.exploration.Explorer;
import mbtest.exploration.ForkExplorer;
import mbtest.exploration.GreedyExplorer;
import mbtest.exploration.LocalGreedyExplorer;
import mbtest.exploration.DFSExplorer;
import mbtest.exploration.StateMachine;
import mbtest.tracing.Logger;

/**
 * @author   Victor Kuliamin
 * @version  0.1
 *
 */
public class FSMExplorerTest
{
  public static void mainx(String[] args) throws Throwable
  {
    StateMachine model = new FSMImpl3(3);
    Explorer explorer  = new OldDFSExplorer();
    
    explorer.run(model);
  }

  public static void mainw(String[] args) throws Throwable
  {
    Logger.setGlobalLogLevel(Logger.DEBUG);
    
    FSMImpl2 model    = new FSMImpl2(101, 21, 15);
    Explorer explorer1 = new OldBFSExplorer();
    
    explorer1.run(model);
    System.out.println(explorer1.getPathLength());
  }

  public static void mainz(String[] args) throws Throwable
  {
    Logger.setGlobalLogLevel(Logger.WARNING);
    
    FSMImpl2 model    = new FSMImpl2(101, 3, 3);
    Explorer explorer1 = new OldDFSExplorer();
    Explorer explorer2 = new DFSExplorer();
    Explorer explorer3 = new GreedyExplorer();
    
    explorer1.run(model);
    explorer2.run(model);
    explorer3.run(model);
    
    System.out.println(explorer1.getPathLength());
    System.out.println(explorer2.getPathLength());
    System.out.println(explorer3.getPathLength());
  }

  public static void mainAA(String[] args) throws Throwable
  {
    Logger.setGlobalLogLevel(Logger.DEBUG);
    
    //StateMachine model = new FSMImpl2(101, 21, 45);
    StateMachine model = new FSMImpl2(7, 2, 2);
    Explorer explorer  = new ForkExplorer(5);//new GreedyExplorer();
    
    explorer.run(model);
    System.out.println(explorer.getPathLength());
  }
  
  
  public static void mainX(String[] args) throws Throwable
  {
    Logger.setGlobalLogLevel(Logger.DEBUG);
    
    //StateMachine model = new FSMImpl2(101, 21, 45);
    StateMachine model = new NDCubeFSM(5);
    Explorer explorer  = new LocalGreedyExplorer();
    
    explorer.run(model);
    System.out.println(explorer.getPathLength());
  }
  
  public static void main2(String[] args) throws Throwable
  {
    Logger.setGlobalLogLevel(Logger.WARNING);
    
    StateMachine model0 = new FSMImpl2(99, 1, 0);
    StateMachine model1 = new FSMImpl2(55, 23, 19);
    StateMachine model2 = new FSMEmpty(0);
    StateMachine model3 = new FSMImpl1(100, 20);
    StateMachine model4 = new FSMImpl3(17);
    StateMachine model5 = new FSMEmpty(10);
    StateMachine model6 = new FSMND(7, 3);
    StateMachine model7 = new FSMNC(11, 3);
    Explorer explorer   = new OldDFSExplorer();
    
    explorer.run(model0);
    explorer.run(model1);
    explorer.run(model2);
    explorer.run(model3);
    explorer.run(model4);
    explorer.run(model5);
    explorer.run(model6);
    explorer.run(model7);
    System.out.println(explorer.getInitialState());
    System.out.println(explorer.getPathLength());
  }
  
  /**
   * @param args
   * @throws Throwable 
   */
  public static void main(String[] args) throws Throwable
  {
    Logger.setGlobalLogLevel(Logger.WARNING);
    
    int[][] res = new int[23][18];
    for(int i = 12; i <= 100; i += 4)
    {
      for(int j = 3; j <= 20; j++)
      {
        FSMImpl1 model    = new FSMImpl1(i-1, j);
        Explorer explorer = new OldDFSExplorer();
        
        explorer.run(model);
        
       // System.out.println("==============================");
       /* System.out.println(  "Number of states : " + explorer.getTotalStates() 
                           + ", state degree : " + (j+1) 
                           + ", path length: " + explorer.getTotalLength());*/
        res[i/4-3][j-3] = explorer.getPathLength();
      }
    }
    
    for(int j = 3; j <= 20; j++) System.out.print(" " + j);
    System.out.println();
    
    for(int i = 12; i <= 100; i += 4)
    {
      System.out.print("" + i + " ");
      for(int j = 3; j <= 20; j++) System.out.print("" + res[i/4-3][j-3] + " ");
      System.out.println();
    }
  }

  public static void mainC(String[] args) throws Throwable
  {
    Logger.setGlobalLogLevel(Logger.WARNING);
    
    int[][][] res = new int[3][30][30];
    for(int i = 15; i < 45; i++)
    {
      for(int j = 2; j < 32; j++)
      {
        StateMachine model = new FSMImpl4(i-1, j);
        Explorer explorer1  = new LocalGreedyExplorer();//BFSExplorer();
        Explorer explorer2  = new GreedyExplorer();//BFSExplorer();
        Explorer explorer3  = new OldBFSExplorer();//BFSExplorer();
        
        explorer1.run(model);
        explorer2.run(model);
        explorer3.run(model);
        
       // System.out.println("==============================");
       /* System.out.println(  "Number of states : " + explorer.getTotalStates() 
                           + ", state degree : " + (j+1) 
                           + ", path length: " + explorer.getTotalLength());*/
        res[0][i-15][j-2] = explorer1.getPathLength();
        res[1][i-15][j-2] = explorer2.getPathLength();
        res[3][i-15][j-2] = explorer3.getPathLength();
      }
    }
    
    for(int j = 2; j < 32; j++) System.out.print(" " + j);
    System.out.println();
    
    for(int i = 15; i < 45; i++)
    {
      System.out.print("" + i + " ");
      for(int j = 2; j < 32; j++)
        System.out.print("" + res[0][i-15][j-2] + " ");
      System.out.println();
      System.out.print("" + i + " ");
      for(int j = 2; j < 32; j++)
        System.out.print("" + res[1][i-15][j-2] + " ");
      System.out.println();
      System.out.print("" + i + " ");
      for(int j = 2; j < 32; j++)
        System.out.print("" + res[2][i-15][j-2] + " ");
      System.out.println();
    }
  }
}
