/**
 * Copyright (c) 2008 ISP RAS.
 * 109004, A. Solzhenitsina, 25, Moscow, Russia.
 * All rights reserved.
 *
 * $Id$
 * Created on 09.05.2013.
 *
 */
package mbtest.tests.guide.intset;

import java.util.HashSet;
import java.util.Set;

import mbtest.contracts.Contract;

/**
 * @author Victor Kuliamin
 *
 */
public class IntSetContract
{
  Set<Integer> set = new HashSet<Integer>();
  
  public boolean sizePost()
  {
    return Contract.assertEquals(Contract.intResult(), set.size()                                
                  , "Size should be equal to the model size")
        && Contract.assertEquals(set, Contract.<Object>oldValue(((HashSet<Integer>)set).clone())
                  , "The set should not change");
                  
  }
  
  public boolean containsPost(int i)
  {
    return Contract.assertEquals(Contract.booleanResult(), set.contains(i)                                
                  , "Result of contains() should be equal to the model contains()")
        && Contract.assertEquals(set, Contract.<Object>oldValue(((HashSet<Integer>)set).clone())
                  , "The set should not change");
                  
  }
  
  public boolean addPost(int i)
  {
    if(Contract.<Boolean>oldValue(set.contains(i)))
    {
      return Contract.assertFalse(Contract.booleanResult()                                 
                  , "Result of add() for contained value should be false")
          && Contract.assertEquals(set, Contract.<Object>oldValue(((HashSet<Integer>)set).clone())
                  , "The set should not change");
    }
    else
    {
      return Contract.assertTrue(Contract.booleanResult()                                 
                  , "Result of add() for non-contained value should be true")
          && Contract.assertTrue(set.contains(i)                                 
                  , "Resulting set should contain the added value")
          && Contract.assertEquals(set.size(), Contract.<Integer>oldValue(set.size())+1
                  , "The size should increase by one");
    }
  }
  
  public boolean removePost(int i)
  {
    if(Contract.<Boolean>oldValue(set.contains(i)))
    {
      return Contract.assertTrue(Contract.booleanResult()                                 
                  , "Result of remove() for contained value should be true")
          && Contract.assertFalse(set.contains(i)                                 
                  , "Resulting set should not contain the removed value")
          && Contract.assertEquals(set.size(), Contract.<Integer>oldValue(set.size())-1
                  , "The size should decrease by one");
    }
    else
    {
      return Contract.assertFalse(Contract.booleanResult()                                 
                  , "Result of remove() for non-contained value should be false")
          && Contract.assertEquals(set, Contract.<Object>oldValue(((HashSet<Integer>)set).clone())
                  , "The set should not change");
    }
  }
  
  public void addUpdate(int i)
  {
    if(!set.contains(i)) set.add(i);
  }

  public void removeUpdate(int i)
  {
    if(set.contains(i)) set.remove(i);
  }
  
}
