/**
 * Copyright (c) 2009 ISP RAS.
 * 109004, A. Solzhenitsina, 25, Moscow, Russia.
 * All rights reserved.
 *
 * $Id$
 * Created on May 8, 2013
 *
 */

package mbtest.tests.guide.list;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import mbtest.annotations.AfterTest;
import mbtest.annotations.DataProvider;
import mbtest.annotations.Guard;
import mbtest.annotations.State;
import mbtest.annotations.Test;
import mbtest.contracts.Contract;

/**
 * @author Victor Kuliamin
 *
 */
@Test
public class ArrayListTest
{
  ArrayList<Integer> list  = new ArrayList<Integer>();
  List<Integer> model = new LinkedList<Integer>();
  int capacity = 0;
  //boolean capacityEnsured = false;
  
  protected int maxValue;
  protected int maxLength;
  
  int N = 20;

  int[] values;

  List<Double> normalData = new LinkedList<Double>();
  List<Double> excessData = new LinkedList<Double>();
  
  int[] indeces()
  {
    int s = model.size();
    int[] r = new int[s];
    for(int i = 0; i < s; i++) r[i] = i;
    
    return r;
  }
  
  boolean sizeBound() { return model.size() < maxLength; }
  
  //boolean capacityAllows() { return model.size() < capacity; }
  
  //boolean capacityExceed() { return model.size() == capacity && capacityEnsured; }
  
  public ArrayListTest() 
  {
    this(2, 2);
  }
  
  public ArrayListTest(int maxValue, int maxLength) 
  {
    this.maxValue  = maxValue;
    this.maxLength = maxLength;
    
    values = new int[maxValue]; 
    for(int i = 0; i < maxValue; i++) values[i] = i;
    
    list.trimToSize();
    //capacityEnsured = true;
  }
  
  @State
  public Object size() { return model.size(); }

  @State
  public Object capacity() { return capacity; }
  
  //@State
  //public boolean capcityEnsured() { return capacityEnsured; } 
  
  @Test
  @DataProvider(name="indeces")
  public void remove(int i)
  {
    Contract.assertEquals(list.remove(i), model.remove(i), "remove() result should be the same as for model");
    Contract.assertEquals(list, model, "Contents after remove() should be the same as for model");
  }
  
  @Test
  @DataProvider(name="indeces")
  public void get(int i)
  {
    Contract.assertEquals(list.get(i), model.get(i), "get() result should be the same as for model");
    Contract.assertEquals(list, model, "Contents after get() should be the same as for model");
  }
  
  @Test
  @Guard(names = "sizeBound")
  public void addInternal(@DataProvider(name="indeces") int i, @DataProvider(name="values") int j)
  {
    list.add(i, j);
    model.add(i, j);
    //if(capacity == model.size()) capacityEnsured = false;
    Contract.assertEquals(list, model, "Contents after internal add() should be the same as for model");
  }

  @Test
  @Guard(names = "sizeBound")
  @DataProvider(name="values")
  public void addLast(int i)
  {
    list.add(i);
    model.add(i);
    //if(capacity == model.size()) capacityEnsured = false;
    Contract.assertEquals(list, model, "Contents after add() in the end should be the same as for model");
  }
  
  @Test
  @Guard(names = "sizeBound")
  @DataProvider(name="values")
  public void addNormal(int i)
  {
    list.ensureCapacity(list.size()+N);
    
    long t = System.nanoTime();
    for(int j = 0; j < N; j++)
      list.add(i);
    
    t = System.nanoTime()-t;
    normalData.add(((double)t)/N);
    
    for(int j = 0; j < N; j++)
      model.add(i);
    
    capacity = model.size();
    //capacityEnsured = false;
    Contract.assertEquals(list, model, "Contents after internal add() should be the same as for model");
  }

  @Test
  @Guard(names = "sizeBound")
  @DataProvider(name="values")
  public void addExceed(int i)
  {
    list.trimToSize();
    
    long t, s = 0;
    for(int j = 0; j < N; j++)
    {
      t = System.nanoTime();
      list.add(i);
      s += System.nanoTime()-t;
      list.trimToSize();
    }
    
    excessData.add(((double)s)/N);
    
    for(int j = 0; j < N; j++)
      model.add(i);
    
    capacity = model.size();
    //capacityEnsured = true;
    Contract.assertEquals(list, model, "Contents after add() in the end should be the same as for model");
  }
  
  @Test
  @DataProvider(name="adds")
  public void testEnsureCapacity(int shift)
  {
    list.ensureCapacity(model.size() + shift);
    capacity = model.size() + shift;
  }
  
  int[] adds = new int[]{0, 1, 5, 10};
  
  @Test
  public void testTrimToSize()
  {
    list.trimToSize();
    capacity = model.size();
    //capacityEnsured = true;
  }
  
  @AfterTest
  public void analyzeData()
  {
    StatData n = new StatData(normalData);
    StatData e = new StatData(excessData);
    System.out.println("Results: normal: " + n);
    System.out.println("Results: excess: " + e);
  }
  
  public static class StatData
  {
    int    number;
    double average;
    //double dispersion;
    double median;
    double firstQuartile;
    double lastQuartile;
    double max;
    double min;
    double firstDecile;
    double lastDecile;
    
    public StatData(List<Double> d)
    {
      number = d.size();
      
      double r = 0;
      for(int i = 0; i < number; i++) r += d.get(i);
      
      average = r/number;
      
      Collections.sort(d);
      
      if(number > 0)
      {
        min = d.get(0);
        max = d.get(number-1);
        median = d.get((number-1)/2);
        firstQuartile = d.get((number-1)/4);
        lastQuartile  = d.get((number-1)-number/4);
        firstDecile   = d.get((number-1)/10);
        lastDecile    = d.get((number-1)-number/10);
      }

    }
    
    public String toString()
    {
      StringBuilder b = new StringBuilder();
      
      b.append("[").append(number).append(",").append(average)
       //.append(",").append(dispersion)
       .append("]{")
       .append(min).append(",")
       .append(firstDecile).append(",")
       .append(firstQuartile).append(",").append(median)
       .append(",").append(lastQuartile).append(",")
       .append(lastDecile).append(",")
       .append(max).append("}");
      
      return b.toString();
    }
  }
  
  double average(long[] d)
  {
    double r = 0;
    int l = d.length;
    for(int i = 0; i < l; i++) r += d[i];
    
    return r/l;
  }
  

}
