/**
 * Copyright (c) 2009 ISP RAS.
 * 109004, A. Solzhenitsina, 25, Moscow, Russia.
 * All rights reserved.
 *
 * $Id$
 * Created on Oct 31, 2012
 *
 */

package mbtest.tests.guide.list;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import mbtest.annotations.DataProvider;
import mbtest.annotations.Guard;
import mbtest.annotations.State;
import mbtest.annotations.Test;
import mbtest.contracts.Contract;

/**
 * @author Victor Kuliamin
 *
 */
@Test
public class ListTest
{
  List<Integer> list  = new ArrayList<Integer>();
  List<Integer> model = new LinkedList<Integer>();
  
  protected int maxValue;
  protected int maxLength;

  int[] values;

  int[] indeces()
  {
    int s = model.size();
    int[] r = new int[s];
    for(int i = 0; i < s; i++) r[i] = i;
    
    return r;
  }
  
  boolean sizeBound() { return model.size() < maxLength; }
  
  public ListTest() 
  {
    this(2, 2);
  }
  
  public ListTest(int maxValue, int maxLength) 
  {
    this.maxValue  = maxValue;
    this.maxLength = maxLength;
    
    values = new int[maxValue]; 
    for(int i = 0; i < maxValue; i++) values[i] = i;
  }
  
  @State
  public Object state() { return ((LinkedList<Integer>)model).clone(); }
  
  @Test
  @DataProvider(name="indeces")
  public void remove(int i)
  {
    Contract.assertEquals(list.remove(i), model.remove(i), "remove() result should be the same as for model");
    Contract.assertEquals(list, model, "Contents after remove() should be the same as for model");
  }
  
  @Test
  @DataProvider(name="indeces")
  public void get(int i)
  {
    Contract.assertEquals(list.get(i), model.get(i), "get() result should be the same as for model");
    Contract.assertEquals(list, model, "Contents after get() should be the same as for model");
  }
  
  @Test
  @Guard(names = "sizeBound")
  public void addInternal(@DataProvider(name="indeces") int i, @DataProvider(name="values") int j)
  {
    list.add(i, j);
    model.add(i, j);
    Contract.assertEquals(list, model, "Contents after internal add() should be the same as for model");
  }

  @Test
  @Guard(names = "sizeBound")
  @DataProvider(name="values")
  public void addLast(int i)
  {
    list.add(model.size(), i);
    model.add(model.size(), i);
    Contract.assertEquals(list, model, "Contents after add() in the end should be the same as for model");
  }
}
