/**
 * Copyright (c) 2009 ISP RAS.
 * 109004, A. Solzhenitsina, 25, Moscow, Russia.
 * All rights reserved.
 *
 * $Id$
 * Created on Oct 25, 2012
 *
 */

package mbtest.tests.guide.stack;

import java.util.LinkedList;
import java.util.List;

import mbtest.annotations.State;
import mbtest.annotations.Test;
import mbtest.contracts.Contract;

/**
 * @author Victor Kuliamin
 *
 */
@Test
public class StackTest
{
  SmallStack stack = new SmallStack();
  
  List<Object> list = new LinkedList<Object>();
  
  @State
  public int getSize() { return stack.size(); } 
  
  @Test(dependsOnMethods="testHead")
  public void testPush()
  {
    int oldSize = stack.size();
    Object o = new Object();
    boolean res = stack.push(o);
    
    if(oldSize < SmallStack.MAX_SIZE)
    {
      Contract.assertTrue(res, "The result should be true");
      Contract.assertEquals(stack.size(), oldSize+1, "Size should increase by 1");
      Contract.assertIdentical(stack.head(), o, "The object pushed should become the head");
      
      list.add(o);
    }
    else
    {
      Contract.assertFalse(res, "The result should be false");
      Contract.assertEquals(stack.size(), oldSize, "Size should not be changed");
      Contract.assertIdentical(stack.head(), list.get(oldSize-1), "The head should not be changed");
    }
  }
  
  @Test(dependsOnMethods="testSize")
  public void testPop()
  {
    int oldSize = stack.size();
    Object res = stack.pop();
    
    if(oldSize > 0)
    {
      Contract.assertEquals(stack.size(), oldSize-1, "Size should decrease by 1");
      Contract.assertIdentical(res, list.get(oldSize-1), "The result should be the old head");
      
      list.remove(oldSize-1);
    }
    else
      Contract.assertEquals(stack.size(), oldSize, "Size should not be changed");
  }
  
  @Test
  public void testHead()
  {
    int oldSize = stack.size();
    
    Contract.assertEquals(stack.size(), oldSize, "Size should not be changed");
    if(oldSize > 0)
      Contract.assertIdentical(stack.head(), list.get(oldSize-1), "The head should not be changed");
    else
      Contract.assertIdentical(stack.head(), null, "The head should be null");
  }
  
  @Test(dependsOnMethods="testPush")
  public void testSize()
  {
    Contract.assertEquals(stack.size(), list.size(), "Size should be equal to the size of stored list");
    Contract.assertEquals(stack.size(), list.size(), "size() call should not change the size");
  }  
}
