/**
 * Copyright (c) 2009 ISP RAS.
 * 109004, A. Solzhenitsina, 25, Moscow, Russia.
 * All rights reserved.
 *
 * $Id$
 * Created on 04.10.2009
 *
 */

package mbtest.tests.maccount;

import mbtest.contracts.Contract;
import mbtest.tracing.Logger;

/**
 * @author Victor Kuliamin
 *
 */
public class AccountContract
{
  static Logger logger = new Logger(AccountContract.class);
  
  int balance;
  int maxCredit;
  
  Account checkedObject;
  
  public void setCheckedObject(Account checkedObject)
  {
    this.checkedObject = checkedObject;
    this.balance       = checkedObject.getBalance();
    this.maxCredit     = checkedObject.getMaxCredit();
  }

  public boolean possibleDeposit(int sum)
  {
    if(sum >= 0 && balance <= Integer.MAX_VALUE - sum) return true;
    else                                               return false;
  }
  
  public boolean possibleWithdraw(int sum)
  {
    if(sum >= 0 && balance >= maxCredit + sum) return true;
    else                                       return false;
  }
  
  boolean checkChangeAbsenceAndZeroResult()
  {
    return    Contract.assertEquals(Contract.intResult(), 0                              
                , "Result should be 0")
           && Contract.assertEquals(balance, Contract.oldIntValue(balance)  
                , "Balance should not change")
           && Contract.assertEquals(maxCredit, Contract.oldIntValue(maxCredit)
                , "Max credit should not change");
  }
  
  public boolean depositPost(int sum)
  {
    if (Contract.oldBooleanValue(possibleDeposit(sum)))
      return    Contract.assertEquals(Contract.intResult(), sum                                
                  , "Result should be equal to the argument")
             && Contract.assertEquals(balance, Contract.oldIntValue(balance) + sum
                  , "Balance should be increased on the argument")
             && Contract.assertEquals(maxCredit, Contract.oldIntValue(maxCredit)    
                  , "Max credit should not change");
    else
      return checkChangeAbsenceAndZeroResult();
  }
  
  public void depositUpdate(int sum)
  {
    if(possibleDeposit(sum))
      balance += sum;
  }

  public boolean withdrawPost(int sum)
  {
    if (Contract.oldBooleanValue(possibleWithdraw(sum)))
      return    Contract.assertEquals(Contract.intResult(), sum                                
                  , "Result should be equal to the argument")
             && Contract.assertEquals(balance, Contract.oldIntValue(balance) - sum
                  , "Balance should be decreased on the argument")
             && Contract.assertEquals(maxCredit, Contract.oldIntValue(maxCredit)    
                  , "Max credit should not change");
    else
      return checkChangeAbsenceAndZeroResult();
  }
  
  public void withdrawUpdate(int sum)
  {
    if(possibleWithdraw(sum))
      balance -= sum;
  }
}
